# Creating Single File JSON Canary

For a basic walkthrough of convention, we will create two, independent, single step, unauthenticated canaries which make HTTP requests.

1. Create the `.cc.json` file. It has all the definitions for the canaries to be generated.
2. Define the version of the Canary.
3. Define the required environment variables to be injected in the container run in `env_var_injection_map`, please define them as `"Key":"Value"` pairs.

    > Note: `Key` has to be lower case and separated by `_` between each word, `Value` has to be upper case and separated by `_` between each word.
    > Each Canary will need some of these environment variables.

4. Each Canary in the `canaries` array is defined as an object, object structure as follows:
    - `machine_readable_name` is the Canary Type and also the machine readable name that appears in the UI, has to be upper case and separated by `_` between each word.
    - `return_data_type` is the returned data type which will be stored, can be any of: `"boolean", "string", "integer", and "decimal"`.
    - `env_vars_schema` is the schema of the environment variables that will be rendered in the UI, schema structure as follows:
        - `type` is the type of the schema.
        - `required` is the array of the required environment variables for this canary to run.
        - `properties` is the object that describes each environment variable and its properties, each object key is the name of the variable and it is defined as an object, the variable object structure as follows:
            - `type` is the type of the environment variable.
            - `title` is the title of the environment variable.
5. Create the `AbstractCanary.py` file. This file defines the actual work of the canary:
    - Copy the file content from the [AbstractCanary.py](HTTPCanary/AbstractCanary.py).
    - Define the base Canary Class to be named according to the canary you want `HTTPCanary`, it will inherit from `CanaryBasePrototype`
    - Each canary class defines two primary metods: `get_response` which is inherited from the base Canary Class, and `parse_response`.
      - These methods are designed to be overwritten.
      - Notice in the `HTTPGetResponseBodyTextCanary` we want to extract `TARGET_VALUE` from an environment variable.
    - Each canary should return the value it is supposed to `"We defined this in step 4 > return_data_type"`.
    - In this example, we map the injected variable `TARGET_VALUE` to the UI property `target_value`
6. Map the canaries run using the `if else` condition in python in the `__main__` section in the `AbstractCanary.py` file to run each canary according to its `machine_readable_name` that you defined in step 4 > `machine_readable_name`
