# Canary Template Tool Kit

Cloud Canaries facilitates the parameterization and scheduling of a collection of containers.  The collection is a `Canary`

A group of `Canaries` come from a group of templates called a `Canary Template Library`

The `Library` is a set of files which configure the canaries based on convention.

The conventions are explained in this SDK through one example:

1. Single File Canary in the [HTTPCanary](HTTPCanary) folder which has the two types of configs for JSON and YML.

The related instructions for creating each type of canary configs can be found in their README files which contains all the required configs:

1. [JSONCanaries](JSONCanaryREADME.md)
2. [YMLCanaries](YMLCanaryREADME.md)

The set of files is then zipped up into a file and uploaded on the `My Templates Dashboard` from the Cloud Canaries Platform via the `+ Create` Button.
> When zipping the files, please consider using either JSON or YML config files.

## To upload

- On the `My Templates` Dashboard, select `+ Create` button
- On the `Upload Canary Template Library` modal:
  - Enter Provider Name (you can choose an older provider from the list or select a new provider)
  - Enter the Service Name
  - Select the zip file via the `Choose File` button
  - Click the `UPLOAD` button

The `Loading` batch operation will take approximately 10-15 minutes to process and generate canaries and build the docker-image.

- Once complete, the `Template` name will appear in the `Service Provider` dropdown via the `+ Create` button on the `Canaries Dashboard`.

## Abstract Canary Prototype Interface

A Prototype `Canary` defines a few public methods

`get_response` performs the work of the canary

`parse_response` transforms the response into the format that Cloud Canaries API expects it

> Note: these methods are to be overwritten by the actual `Canary` to be created.

## Sending data to the Cloud Canaries API server

The Canary then reports back the transformed response to the API server in the format:

All of these fields are automatically constructed and passed when the conventions of Cloud Canaries are followed.

Authentication API keys are automatically injected via environmental variables.

```json
   {
     "error_get": Boolean,
     "error_parse": Boolean,
     "value": Any,
     "response_latency": Integer,
     "canary_id": String,
     "dag_id": String,
     "task_id": String,
     "run_id": String,
     "run_try_number": Integer,
     "measured": Datetime
   }
   
   headers = {'Authorization': 'Api-Key %s' % self._api_key}
   
```

## Folder Structure

File descriptions are available in each file.

All files must be present in order for Canary Template Libraries to Populate properly

## Tutorial

- [JSON file tutorial](JSONCanaryREADME.md)
- [YML file tutorial](YMLCanaryREADME.md)

---
Copyright © 2023 Cloud Canaries Inc.
